﻿using System.Xml;

namespace StellarWaterPoint.WaterBills
{
    public partial class Create : Form
    {
        public Create()
        {
            InitializeComponent();
        }

        private void btnFindCustomerAccount_Click(object sender, EventArgs e)
        {
            string? strMeterNumber = null;
            XmlDocument xdCustomers = new XmlDocument();
            string? strCustomers = @"C:\Stellar Water Point\Customers.xml";

            FileInfo? fiCustomers = new FileInfo(strCustomers);

            if (fiCustomers.Exists)
            {
                using (FileStream? fsCustomers = new FileStream(fiCustomers.FullName, FileMode.Open, FileAccess.Read, FileShare.Read))
                {
                    xdCustomers.Load(fsCustomers);
                    XmlNodeList xnlCustomers = xdCustomers.DocumentElement!.SelectNodes("//account-number[.='" + mtbAccountNumber.Text + "']")!;

                    foreach (XmlNode xnCustomer in xnlCustomers)
                    {
                        strMeterNumber       = xnCustomer.NextSibling!.InnerText;   // Meter #
                        txtCustomerName.Text = xnCustomer.NextSibling.NextSibling!.InnerText + " " +
                                               xnCustomer.NextSibling.NextSibling.NextSibling!.InnerText;   // Customer Name
                        txtAddress.Text      = xnCustomer.NextSibling.NextSibling.NextSibling.NextSibling!.InnerText;   // Address
                        txtCity.Text         = xnCustomer.NextSibling.NextSibling.NextSibling.NextSibling!.NextSibling!.InnerText;   // City
                        txtCounty.Text       = xnCustomer.NextSibling.NextSibling.NextSibling.NextSibling!.NextSibling!.NextSibling!.InnerText;   // County
                        txtState.Text        = xnCustomer.NextSibling.NextSibling.NextSibling.NextSibling!.NextSibling!.NextSibling!.NextSibling!.InnerText;   // State
                        mtbZIPCode.Text      = xnCustomer.NextSibling.NextSibling.NextSibling.NextSibling!.NextSibling!.NextSibling!.NextSibling!.NextSibling!.InnerText; // ZIP-Code
                    }
                }
            }

            XmlDocument xdWaterMeters  = new XmlDocument();
            string?     strWaterMeters = @"C:\Stellar Water Point\WaterMeters.xml";

            FileInfo?   fiWaterMeters  = new FileInfo(strWaterMeters);

            if (fiWaterMeters.Exists)
            {
                using (FileStream? fsWaterMeters = new FileStream(fiWaterMeters.FullName, FileMode.Open, FileAccess.Read, FileShare.Read))
                {
                    xdWaterMeters.Load(fsWaterMeters);
                    XmlNodeList xnlWaterMeters = xdWaterMeters.DocumentElement!.SelectNodes("//meter-number[.='" + strMeterNumber + "']")!;

                    foreach (XmlNode xnWaterMeter in xnlWaterMeters)
                    {
                        txtMeterDetails.Text = xnWaterMeter.NextSibling!.InnerText + " " +
                                               xnWaterMeter.NextSibling.NextSibling!.InnerText + " (Meter Size: " +
                                               xnWaterMeter.NextSibling.NextSibling.NextSibling!.InnerText + ")";
                    }
                }
            }
        }

        private void dtpMeterReadingEndDate_ValueChanged(object sender, EventArgs e)
        {
            TimeSpan tsDays = dtpMeterReadingEndDate.Value - dtpMeterReadingStartDate.Value;

            txtBillingDays.Text = tsDays.Days.ToString();
        }

        private void btnEvaluateWaterBill_Click(object sender, EventArgs e)
        {
            double counterStart = 0, counterEnd = 0;

            try
            {
                counterStart = double.Parse(txtCounterReadingStart.Text);
            }
            catch (FormatException feCRStart)
            {
                MessageBox.Show("There was a problem with the value of the " +
                                "Counter Reading Start. The error produced is: " + feCRStart.Message,
                                "Stellar Water Point", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }

            try
            {
                counterEnd = double.Parse(txtCounterReadingEnd.Text);
            }
            catch (FormatException feCREnd)
            {
                MessageBox.Show("There was a problem with the value of the " +
                                "Counter Reading End. The error produced is: " + feCREnd.Message,
                                "Stellar Water Point", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }

            double consumption  = counterEnd   - counterStart;
            double gallons      = consumption  * 748.05;
            double firstTier    = gallons      * (48.00 / 10000.00);
            double secondTier   = gallons      * (32.00 / 10000.00);
            double lastTier     = gallons      * (20.00 / 10000.00);
            double waterCharges = firstTier    + secondTier + lastTier;
            double sewerCharges = waterCharges * 28.65 / 100;
            double envCharges   = waterCharges * 0.22184;
            double totalCharges = waterCharges + sewerCharges + envCharges;
            double localTaxes   = totalCharges * 0.06148;
            double stateTaxes   = totalCharges * 0.01374;
            double amtDue       = totalCharges + localTaxes + stateTaxes;

            TimeSpan tsPaymentDueDate = new TimeSpan(15, 0, 0, 0);

            txtTotalCCF.Text              = consumption.ToString();
            txtTotalGallons.Text          = gallons.ToString("F");
            txtFirstTierConsumption.Text  = firstTier.ToString("F");
            txtSecondTierConsumption.Text = secondTier.ToString("F");
            txtLastTierConsumption.Text   = lastTier.ToString("F");
            txtWaterCharges.Text          = waterCharges.ToString("F");
            txtSewerCharges.Text          = sewerCharges.ToString("F");
            txtEnvironmentCharges.Text    = envCharges.ToString("F");
            txtTotalCharges.Text          = totalCharges.ToString("F");
            txtLocalTaxes.Text            = localTaxes.ToString("F");
            txtStateTaxes.Text            = stateTaxes.ToString("F");
            dtpPaymentDueDate.Value       = dtpMeterReadingEndDate.Value + tsPaymentDueDate;
            txtAmountDue.Text             = amtDue.ToString("F");
            dtpLatePaymentDueDate.Value   = dtpMeterReadingEndDate.Value + new TimeSpan(30, 0, 0, 0);
            txtLateAmountDue.Text         = (amtDue + 8.95).ToString("F");
        }

        private void btnSaveWaterBill_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(txtBillNumber.Text))
            {
                MessageBox.Show("You must type a bill number." +
                                "Otherwise, the account cannot be saved.",
                                "Stellar Water Point", MessageBoxButtons.OK);
                return;
            }

            if (string.IsNullOrEmpty(txtCounterReadingStart.Text))
            {
                MessageBox.Show("You must enter the start value of the water bill counter." +
                                "Otherwise, the account cannot be saved.",
                                "Stellar Water Point", MessageBoxButtons.OK);
                return;
            }

            if (string.IsNullOrEmpty(mtbAccountNumber.Text))
            {
                MessageBox.Show("You must type an account number of a customer." +
                                "Otherwise, the account cannot be saved.",
                                "Stellar Water Point", MessageBoxButtons.OK);
                return;
            }

            FileStream? fsWaterBills = null;
            XmlDocument xdWaterBills = new XmlDocument();
            string strWaterBills = @"C:\Stellar Water Point\WaterBills.xml";

            FileInfo? fiWaterBills = new FileInfo(strWaterBills);
            
            if (fiWaterBills.Exists)    
            {
                using(fsWaterBills = new FileStream(fiWaterBills.FullName, FileMode.Open, FileAccess.Read, FileShare.Read))
                {
                    xdWaterBills.Load(fsWaterBills);
                }
            }
            else
            {
                using (fsWaterBills = new FileStream(fiWaterBills.FullName, FileMode.Create, FileAccess.Write, FileShare.Write))
                {
                    xdWaterBills.LoadXml("<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
                                         "<water-bills></water-bills>");
                    xdWaterBills.Save(fsWaterBills);
                }
            }
            
            using (fsWaterBills = new FileStream(fiWaterBills.FullName, FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite))
            {
                XmlElement xeWaterBill = xdWaterBills.CreateElement("water-bill");
                
                xeWaterBill.InnerXml = "<bill-number>"              + txtBillNumber.Text            + "</bill-number>"             + 
                                       "<account-number>"           + mtbAccountNumber.Text         + "</account-number>"          +
                                       "<meter-reading-start-date>" + dtpMeterReadingStartDate.Value.ToShortDateString()           + "</meter-reading-start-date>" +
                                       "<meter-reading-end-d ate>"  + dtpMeterReadingEndDate.Value.ToShortDateString()             + "</meter-reading-end-date>" +
                                       "<billing-days>"             + txtBillingDays.Text           + "</billing-days>"            +
                                       "<counter-reading-start>"    + txtCounterReadingStart.Text   + "</counter-reading-start>"   +
                                       "<counter-reading-end>"      + txtCounterReadingEnd.Text     + "</counter-reading-end>"     +
                                       "<total-gallons>"            + txtTotalGallons.Text          + "</total-gallons>"           +
                                       "<total-ccf>"                + txtTotalCCF.Text              + "</total-ccf>"               +
                                       "<first-tier-consumption>"   + txtFirstTierConsumption.Text  + "</first-tier-consumption>"  +
                                       "<second-tier-consumption>"  + txtSecondTierConsumption.Text + "</second-tier-consumption>" +
                                       "<last-tier-consumption>"    + txtLastTierConsumption.Text   + "</last-tier-consumption>"   +
                                       "<water-charges>"            + txtWaterCharges.Text          + "</water-charges>"           +
                                       "<sewer-charges>"            + txtSewerCharges.Text          + "</sewer-charges>"           +
                                       "<environment-charges>"      + txtEnvironmentCharges.Text    + "</environment-charges>"     +
                                       "<total-charges>"            + txtTotalCharges.Text          + "</total-charges>"           +
                                       "<local-taxes>"              + txtLocalTaxes.Text            + "</local-taxes>"             +
                                       "<state-taxes>"              + txtStateTaxes.Text            + "</state-taxes>"             +
                                       "<amount-due>"               + txtAmountDue.Text             + "</amount-due>"              +
                                       "<payment-due-date>"         + dtpPaymentDueDate.Value.ToShortDateString()                  + "</payment-due-date>" +
                                       "<late-amount-due>"          + txtLateAmountDue.Text         + "</late-amount-due>"         +
                                       "<late-payment-due-date>"    + dtpLatePaymentDueDate.Value.ToShortDateString()              + "</late-payment-due-date>";
                
                xdWaterBills.DocumentElement!.AppendChild(xeWaterBill);
                xdWaterBills.Save(fsWaterBills);
            }

            Close();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
